"use strict";
Object.defineProperty(exports, Symbol.toStringTag, { value: "Module" });
const view = require("@codemirror/view");
const React = require("react");
const shim = require("use-sync-external-store/shim");
const jsxRuntime = require("react/jsx-runtime");
function isFunction(value) {
  return typeof value === "function";
}
function useSingleton(instance) {
  const instanceRef = React.useRef(null);
  if (instanceRef.current === null) {
    instanceRef.current = isFunction(instance) ? instance() : instance;
  }
  return instanceRef.current;
}
const isBrowser = typeof window !== "undefined";
const builtInAPI = React.useInsertionEffect;
function syncFallback(effect, _deps) {
  effect();
}
const useInsertionEffectWithSyncFallback = isBrowser ? builtInAPI != null ? builtInAPI : syncFallback : syncFallback;
function useSyncedRef(value) {
  const ref = React.useRef(value);
  useInsertionEffectWithSyncFallback(() => {
    ref.current = value;
  }, [value]);
  return ref;
}
function useRefWithEffect(initialValue, effect) {
  const effectRef = useSyncedRef(effect);
  return useSingleton(() => {
    let currentValue = initialValue;
    let cleanup;
    return {
      get current() {
        return currentValue;
      },
      set current(value) {
        if (value === currentValue) {
          return;
        }
        currentValue = value;
        cleanup == null ? void 0 : cleanup();
        const callback = effectRef.current;
        cleanup = callback(value);
      }
    };
  });
}
function createCodeMirror(config) {
  let currentView;
  const viewUpdateCallbacks = /* @__PURE__ */ new Set();
  function subscribeViewUpdate(callback) {
    viewUpdateCallbacks.add(callback);
    return () => {
      viewUpdateCallbacks.delete(callback);
    };
  }
  function publishViewUpdate() {
    viewUpdateCallbacks.forEach((callback) => callback());
  }
  function setView(view2) {
    currentView = view2;
    publishViewUpdate();
  }
  const getView = () => currentView;
  const useView = () => {
    const view2 = shim.useSyncExternalStore(
      subscribeViewUpdate,
      getView,
      () => void 0
    );
    React.useDebugValue(view2);
    return view2;
  };
  const useViewEffect = (effect, deps) => {
    const view2 = useView();
    React.useEffect(() => {
      if (view2) {
        return effect(view2);
      }
    }, [view2, ...deps]);
  };
  let prevState;
  const createConfig = isFunction(config) ? config : () => config;
  const useContainerRef = () => useRefWithEffect(null, (container) => {
    if (!container || typeof window === "undefined") {
      return;
    }
    const view$1 = new view.EditorView({
      ...createConfig(prevState),
      parent: container
    });
    setView(view$1);
    return () => {
      prevState = view$1.state;
      view$1.destroy();
      setView(void 0);
    };
  });
  return {
    getView,
    useView,
    useViewEffect,
    useContainerRef
  };
}
function createCodeMirrorContext() {
  const InternalCodeMirrorContext = React.createContext(null);
  const Provider = ({ config, children }) => {
    const codeMirror = useSingleton(() => createCodeMirror(config));
    return /* @__PURE__ */ jsxRuntime.jsx(InternalCodeMirrorContext.Provider, { value: codeMirror, children });
  };
  const useCodeMirror = () => {
    const codeMirror = React.useContext(InternalCodeMirrorContext);
    if (!codeMirror) {
      throw new Error(
        "could not find CodeMirrorContext value; please ensure the component is wrapped in a <Provider>"
      );
    }
    return codeMirror;
  };
  const useView = () => {
    const { useView: useView2 } = useCodeMirror();
    return useView2();
  };
  const useViewEffect = (effect, deps) => {
    const { useViewEffect: useViewEffect2 } = useCodeMirror();
    return useViewEffect2(effect, deps);
  };
  const useContainerRef = () => {
    const { useContainerRef: useContainerRef2 } = useCodeMirror();
    return useContainerRef2();
  };
  return {
    Provider,
    useCodeMirror,
    useView,
    useViewEffect,
    useContainerRef
  };
}
exports.createCodeMirror = createCodeMirror;
exports.createCodeMirrorContext = createCodeMirrorContext;
