import { EditorView } from "@codemirror/view";
import React, { useRef, useDebugValue, useEffect, createContext, useContext } from "react";
import { useSyncExternalStore } from "use-sync-external-store/shim";
import { jsx } from "react/jsx-runtime";
function isFunction(value) {
  return typeof value === "function";
}
function useSingleton(instance) {
  const instanceRef = useRef(null);
  if (instanceRef.current === null) {
    instanceRef.current = isFunction(instance) ? instance() : instance;
  }
  return instanceRef.current;
}
const isBrowser = typeof window !== "undefined";
const builtInAPI = React.useInsertionEffect;
function syncFallback(effect, _deps) {
  effect();
}
const useInsertionEffectWithSyncFallback = isBrowser ? builtInAPI != null ? builtInAPI : syncFallback : syncFallback;
function useSyncedRef(value) {
  const ref = useRef(value);
  useInsertionEffectWithSyncFallback(() => {
    ref.current = value;
  }, [value]);
  return ref;
}
function useRefWithEffect(initialValue, effect) {
  const effectRef = useSyncedRef(effect);
  return useSingleton(() => {
    let currentValue = initialValue;
    let cleanup;
    return {
      get current() {
        return currentValue;
      },
      set current(value) {
        if (value === currentValue) {
          return;
        }
        currentValue = value;
        cleanup == null ? void 0 : cleanup();
        const callback = effectRef.current;
        cleanup = callback(value);
      }
    };
  });
}
function createCodeMirror(config) {
  let currentView;
  const viewUpdateCallbacks = /* @__PURE__ */ new Set();
  function subscribeViewUpdate(callback) {
    viewUpdateCallbacks.add(callback);
    return () => {
      viewUpdateCallbacks.delete(callback);
    };
  }
  function publishViewUpdate() {
    viewUpdateCallbacks.forEach((callback) => callback());
  }
  function setView(view) {
    currentView = view;
    publishViewUpdate();
  }
  const getView = () => currentView;
  const useView = () => {
    const view = useSyncExternalStore(
      subscribeViewUpdate,
      getView,
      () => void 0
    );
    useDebugValue(view);
    return view;
  };
  const useViewEffect = (effect, deps) => {
    const view = useView();
    useEffect(() => {
      if (view) {
        return effect(view);
      }
    }, [view, ...deps]);
  };
  let prevState;
  const createConfig = isFunction(config) ? config : () => config;
  const useContainerRef = () => useRefWithEffect(null, (container) => {
    if (!container || typeof window === "undefined") {
      return;
    }
    const view = new EditorView({
      ...createConfig(prevState),
      parent: container
    });
    setView(view);
    return () => {
      prevState = view.state;
      view.destroy();
      setView(void 0);
    };
  });
  return {
    getView,
    useView,
    useViewEffect,
    useContainerRef
  };
}
function createCodeMirrorContext() {
  const InternalCodeMirrorContext = createContext(null);
  const Provider = ({ config, children }) => {
    const codeMirror = useSingleton(() => createCodeMirror(config));
    return /* @__PURE__ */ jsx(InternalCodeMirrorContext.Provider, { value: codeMirror, children });
  };
  const useCodeMirror = () => {
    const codeMirror = useContext(InternalCodeMirrorContext);
    if (!codeMirror) {
      throw new Error(
        "could not find CodeMirrorContext value; please ensure the component is wrapped in a <Provider>"
      );
    }
    return codeMirror;
  };
  const useView = () => {
    const { useView: useView2 } = useCodeMirror();
    return useView2();
  };
  const useViewEffect = (effect, deps) => {
    const { useViewEffect: useViewEffect2 } = useCodeMirror();
    return useViewEffect2(effect, deps);
  };
  const useContainerRef = () => {
    const { useContainerRef: useContainerRef2 } = useCodeMirror();
    return useContainerRef2();
  };
  return {
    Provider,
    useCodeMirror,
    useView,
    useViewEffect,
    useContainerRef
  };
}
export {
  createCodeMirror,
  createCodeMirrorContext
};
